/*
 * stat.h
 */


#ifndef sys_stat_h
#define sys_stat_h


#include<sys/types.h>


enum{
	/* File type. */
	S_IFMT

	/* File mode bits. */
	S_IRWXU		/* Read, write, execute/search by owner. */
	S_IRUSR		/* Read permission, owner. */
	S_IWUSR		/* Write permission, owner. */
	S_IXUSR		/* Execute/search permission, owner. */
	S_IRWXG		/* Read, write, execute/search by group. */
	S_IRGRP		/* Read permission, group. */
	S_IWGRP		/* Write permission, group. */
	S_IXGRP		/* Execute/search permission, group. */
	S_IRWXO		/* Read, write, execute/search by others. */
	S_IROTH		/* Read permission, others. */
	S_IWOTH		/* Write permission, others. */
	S_IXOTH		/* Execute/search permission, others. */
	S_ISUID		/* Set-user-ID on execution. */
	S_ISGID		/* Set-group-ID on execution. */
	S_IRWXU		/* bitwise-inclusive OR of S_IRUSR, S_IWUSR, and S_IXUSR. */
	S_IRWXG		/* bitwise-inclusive OR of S_IRGRP, S_IWGRP, and S_IXGRP. */
	S_IRWXO		/* bitwise-inclusive OR of S_IROTH, S_IWOTH, and S_IXOTH. */
};


/* The macro shall evaluate to a non-zero value if the test is true; 0 if the test is false. */
#define S_ISBLK(m)		/* Test for a block special file. */
#define S_ISCHR(m)		/* Test for a character special file. */
#define S_ISDIR(m)		/* Test for a directory. */
#define S_ISFIFO(m)		/* Test for a pipe or FIFO special file. */
#define S_ISREG(m)		/* Test for a regular file. */
#define S_ISLNK(m)		/* Test for a symbolic link. */
#define S_ISSOCK(m)		/* Test for a socket. */

/*
 * The implementation may implement message queues, semaphores, or shared memory objects as distinct
 * file types. The following macros shall be provided to test whether a file is of the specified type.
 * The value of the buf argument supplied to the macros is a pointer to a stat structure. The macro
 * shall evaluate to a non-zero value if the specified object is implemented as a distinct file type
 * and the specified file type is contained in the stat structure referenced by buf. Otherwise,
 * the macro shall evaluate to zero.
 */
#define S_TYPEISMQ(buf)			/* Test for a message queue. */
#define S_TYPEISSEM(buf)		/* Test for a semaphore. */
#define S_TYPEISSHM(buf)		/* Test for a shared memory object. */


/* The stat structure. */
struct stat{
	dev_t     st_dev;		/* Device ID of device containing file. */
	ino_t     st_ino;		/* File serial number. */
	mode_t    st_mode;		/* Mode of file. */
	nlink_t   st_nlink;		/* Number of hard links to the file. */
	uid_t     st_uid;		/* User ID of file. */
	gid_t     st_gid;		/* Group ID of file. */
	off_t     st_size;		/* For regular files, the file size in bytes. */
	time_t    st_atime;		/* Time of last access. */
	time_t    st_mtime;		/* Time of last data modification. */
	time_t    st_ctime;		/* Time of last status change. */
};


/* functions and may also be defined as macros. */
int    chmod(const char *, mode_t);
int    fchmod(int, mode_t);
int    fstat(int, struct stat *);
int    lstat(const char *restrict, struct stat *restrict);
int    mkdir(const char *, mode_t);
int    mkfifo(const char *, mode_t);
int    stat(const char *restrict, struct stat *restrict);
mode_t umask(mode_t);


#endif
